@@ -26,16 +26,14 @@ class CSRFMiddleware(BaseHTTPMiddleware):
     is rejected.
 
     This uses the Double Submit Cookie style of CSRF prevention. For more
-    information:
-
-    https://cheatsheetseries.owasp.org/cheatsheets/Cross-Site_Request_Forgery_Prevention_Cheat_Sheet.html#double-submit-cookie
-    https://cheatsheetseries.owasp.org/cheatsheets/Cross-Site_Request_Forgery_Prevention_Cheat_Sheet.html#use-of-custom-request-headers
+    information see the OWASP cheatsheet (`double submit cookie <https://cheatsheetseries.owasp.org/cheatsheets/Cross-Site_Request_Forgery_Prevention_Cheat_Sheet.html#double-submit-cookie>`_
+    and `customer request headers <https://cheatsheetseries.owasp.org/cheatsheets/Cross-Site_Request_Forgery_Prevention_Cheat_Sheet.html#use-of-custom-request-headers>`_).
 
     By default, the CSRF token needs to be added to the request header. By
-    setting `allow_form_param` to True, it will also work if added as a form
-    parameter.
+    setting ``allow_form_param`` to ``True``, it will also work if added as a
+    form parameter.
 
-    """
+    """  # noqa: E501
 
     @staticmethod
     def get_new_token() -> str:
@@ -45,31 +43,31 @@ class CSRFMiddleware(BaseHTTPMiddleware):
         self,
         app: ASGIApp,
         allowed_hosts: t.Sequence[str] = [],
-        cookie_name=DEFAULT_COOKIE_NAME,
-        header_name=DEFAULT_HEADER_NAME,
-        max_age=ONE_YEAR,
-        allow_header_param=True,
-        allow_form_param=False,
+        cookie_name: str = DEFAULT_COOKIE_NAME,
+        header_name: str = DEFAULT_HEADER_NAME,
+        max_age: int = ONE_YEAR,
+        allow_header_param: bool = True,
+        allow_form_param: bool = False,
         **kwargs,
     ):
         """
         :param app:
             The ASGI app you want to wrap.
-        :allowed_hosts:
+        :param allowed_hosts:
             If using this middleware with HTTPS, you need to set this value,
-            for example ['example.com'].
-        :cookie_name:
+            for example ``['example.com']``.
+        :param cookie_name:
             You can specify a custom name for the cookie. There should be no
             need to change it, unless in the rare situation where the name
             clashes with another cookie.
-        :header_name:
+        :param header_name:
             You can tell the middleware to look for the CSRF token in a
             different HTTP header.
-        :max_age:
-            The max age of the cookie.
-        :allow_header_param:
+        :param max_age:
+            The max age of the cookie, in seconds.
+        :param allow_header_param:
             Whether to look for the CSRF token in the HTTP headers.
-        :allow_form_param:
+        :param allow_form_param:
             Whether to look for the CSRF token in a form field with the same
             name as the cookie. By default, it's not enabled.
 
@@ -129,12 +127,13 @@ class CSRFMiddleware(BaseHTTPMiddleware):
             if not cookie_token:
                 return Response("No CSRF cookie found", status_code=403)
 
-            if self.allow_header_param:
-                header_token = request.headers.get(self.header_name, None)
-            else:
-                header_token = None
+            header_token = (
+                request.headers.get(self.header_name)
+                if self.allow_header_param
+                else None
+            )
 
-            if self.allow_form_param:
+            if self.allow_form_param and not header_token:
                 form_data = await request.form()
                 form_token = form_data.get(self.cookie_name, None)
                 request.scope.update({"form": form_data})
