@@ -195,7 +195,7 @@ class SessionLoginEndpoint(HTTPEndpoint, metaclass=ABCMeta):
     async def post(self, request: Request) -> Response:
         # Some middleware (for example CSRF) has already awaited the request
         # body, and adds it to the request.
-        body = request.scope.get("form")
+        body: t.Any = request.scope.get("form")
 
         if not body:
             try:
@@ -379,7 +379,9 @@ def session_login(
     )
 
     directory, filename = os.path.split(template_path)
-    environment = Environment(loader=FileSystemLoader(directory))
+    environment = Environment(
+        loader=FileSystemLoader(directory), autoescape=True
+    )
     login_template = environment.get_template(filename)
 
     class _SessionLoginEndpoint(SessionLoginEndpoint):
@@ -431,7 +433,9 @@ def session_logout(
     )
 
     directory, filename = os.path.split(template_path)
-    environment = Environment(loader=FileSystemLoader(directory))
+    environment = Environment(
+        loader=FileSystemLoader(directory), autoescape=True
+    )
     logout_template = environment.get_template(filename)
 
     class _SessionLogoutEndpoint(SessionLogoutEndpoint):
